
/*
 * Fast implementation of the DES, as described in the Federal Register,
 * Vol. 40, No. 52, p. 12134, March 17, 1975.
 *
 * Stuart Levy, Minnesota Supercomputer Center, April 1988.
 *
 * Compilation:
 * There's one performance-determining compilation switch, -DSLOWSHIFT.
 * If defined, the code avoids 32-bit shift operations, preferring instead
 * to extract 8- and 16-bit quantities from memory.  This may help for 16-bit
 * processors or those which do not do shifts efficiently.  It hurts for
 * 32-bit CPUs with barrel shifters, e.g. the 68020.  Try it and see.
 *
 * If you -DSLOWSHIFT you must also define -DENDIAN=xxx where "xxx" is either
 *    BIG    for machines where the MSByte is the low address: 68000, ...
 * or LITTLE for machines where the LSByte is the low addr: VAX, 8086, ...
 *
 * If you get it wrong fsetkey() will print a message and dump core at run time.
 *
 * Calling sequence:
 *
 * typedef unsigned long keysched[32];
 *
 * fsetkey(key, keysched)	/ * Converts a DES key to a "key schedule" * /
 *	unsigned byte	key[8];
 *	keysched	*ks;
 *
 * fencrypt(block, decrypt, keysched)	/ * En/decrypts one 64-bit block * /
 *	unsigned byte	block[8];	/ * data, en/decrypted in place * /
 *	int		decrypt;	/ * 0=>encrypt, 1=>decrypt * /
 *	keysched	*ks;		/ * key schedule, as set by fsetkey * /
 *
 * Key and data block representation:
 * The 56-bit key (bits 1..64 including "parity" bits 8, 16, 24, ..., 64)
 * and the 64-bit data block (bits 1..64)
 * are each stored in arrays of 8 bytes.
 * Following the NBS numbering, the MSB has the bit number 1, so
 *  key[0] = 128*bit1 + 64*bit2 + ... + 1*bit8, ... through
 *  key[7] = 128*bit57 + 64*bit58 + ... + 1*bit64.
 * In the key, "parity" bits are not checked; their values are ignored.
 */

/*
typedef unsigned long word32;
typedef unsigned char byte;
*/
#include "usuals.h"  /* typedefs for byte, word16, boolean, etc. */

typedef struct keysched {
	struct keystage {
		word32 h, l;
	} KS[16];
} keysched;


void fsetkey(byte key[8], keysched *ks);	/* Converts a DES key to a "key schedule" */
void fencrypt(byte block[8], int decrypt, keysched *ks);	/* En/decrypts one 64-bit block */


#undef SLOWSHIFT


/*##########################-3DES-##########################*/

typedef keysched keysched3[3];

int fsetkey3(byte key[8*3], keysched3 *ks);
void fencrypt3(byte block[8], int decrypt, keysched3 *ks);

/*##########################-3DES-##########################*/
